#include <errno.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>

#include <mosquitto.h>

void mosqOnConnect ( struct mosquitto* mosq, void* ud, int rc );
void mosqOnDisconnect ( struct mosquitto* mosq, void* ud, int rc );
void mosqOnPublish ( struct mosquitto* mosq, void* ud, int mid );
void mosqOnSubscribe ( struct mosquitto* mosq, void* ud, int mid, int qos_count, const int* granted_qos );
void mosqOnUnsubscribe ( struct mosquitto* mosq, void* ud, int mid );
void mosqOnMessage ( struct mosquitto* mosq, void* ud, const struct mosquitto_message* msg );

int running = 1;

int main ( int argc, char** argv )
{
	--argc; ++argv;
	if( argc != 1 ) { return 1; }

	struct mosquitto* mosq = NULL;
	mosquitto_lib_init();
	mosq = mosquitto_new( NULL, 1, NULL );
	mosquitto_connect_callback_set( mosq, mosqOnConnect );
	mosquitto_disconnect_callback_set( mosq, mosqOnDisconnect );
	mosquitto_publish_callback_set( mosq, mosqOnPublish );
	mosquitto_subscribe_callback_set( mosq, mosqOnSubscribe );
	mosquitto_unsubscribe_callback_set( mosq, mosqOnUnsubscribe );
	mosquitto_message_callback_set( mosq, mosqOnMessage );
	mosquitto_loop_start( mosq );
	usleep( 250000 );
	mosquitto_connect( mosq, "localhost", 1883, 1 );
	usleep( 250000 );

	int rc = mosquitto_subscribe( mosq, NULL, *argv, 0 );
	printf( "sub '%s' = %d\n", *argv, rc );
	fflush( stdout );

	while( running ) {
		printf( "waiting ...\n" );
		fflush( stdout );
		sleep( 1 );
	}

	mosquitto_disconnect( mosq );
	usleep( 250000 );
	mosquitto_loop_stop( mosq, 0 );
	usleep( 250000 );
	mosquitto_lib_cleanup();

	return 0;
}

/** Mosquitto MQTT "on connect" handler */
void mosqOnConnect ( struct mosquitto* mosq, void* ud, int rc )
{
	printf( "OnConnect: ud=%p rc=%d\n", ud, rc );
	fflush( stdout );
}

/** Mosquitto MQTT "on disconnect" handler */
void mosqOnDisconnect ( struct mosquitto* mosq, void* ud, int rc )
{
	printf( "OnDisconnect: ud=%p rc=%d\n", ud, rc );
	fflush( stdout );
}

/** Mosquitto MQTT "on publish" handler */
void mosqOnPublish ( struct mosquitto* mosq, void* ud, int mid )
{
	printf( "OnPublish: ud=%p mid=%d\n", ud, mid );
	fflush( stdout );
}

/** Mosquitto MQTT "on subscribe" handler */
void mosqOnSubscribe ( struct mosquitto* mosq, void* ud, int mid, int qos_count, const int* granted_qos )
{
	printf( "OnSubscribe: ud=%p mid=%d count=%d\n", ud, mid, qos_count );
	fflush( stdout );
}

/** Mosquitto MQTT "on unsubscribe" handler */
void mosqOnUnsubscribe ( struct mosquitto* mosq, void* ud, int mid )
{
	printf( "OnUnsubscribe: ud=%p mid=%d\n", ud, mid );
	fflush( stdout );
}

/** Mosquitto MQTT "on message" handler */
void mosqOnMessage ( struct mosquitto* mosq, void* ud, const struct mosquitto_message* msg )
{
	printf( "OnMessage: ud=%p len=%d message='%.*s'\n", ud, msg->payloadlen, msg->payloadlen, msg->payload );
	fflush( stdout );

	running = !(msg->payloadlen == 1 && *((char*)msg->payload) == '.');
}
